%%% Filename : f_checkclass.pl
%%% Date     : 06.02.2014
%%% Contents : checks the class of the frame condition in prenex nf
%%% Author   : Lara Katharina Spendier <lara@logic.at>

isClasses(X ';' Y, Class) :-
	X = Prefix ':' _,
	isClass(Prefix, C1),
	isClasses(Y, C2),
	append([C1], C2, Class).
isClasses(X, Class) :-
	X = Prefix ':' _,
	isClass(Prefix, C1),
	Class = [C1].

%%% isClass(+Prefix, -Class)
%%%		compute the class of the frame condition according to 
%%%		the quantifier alternation in the prefix
%%%		+Prefix ... prefix of the formula
%%%		-Class  ... Class (sigma/pi) of the formula: s(#) or p(#)
isClass(Prefix, C) :-
	Prefix = 'A' _,
	isPiClass(Prefix, 0, N),
	C = p(N).

isClass(Prefix, C) :-
	Prefix = 'E' _,
	isSigmaClass(Prefix, 0, N),
	C = s(N).

%%% isPiClass(+Prefix, +Class, -Class)
%%%		compute the class level
%%%		
isPiClass(Prefix, P0, P0) :-
	isPiFormula(Prefix, P0).
isPiClass(Prefix, P0, P2) :-
	\+ isPiFormula(Prefix, P0),
	P1 is P0+1,
	isPiClass(Prefix, P1, P2).

isSigmaClass(Prefix, S0, S0) :-
	isSigmaFormula(Prefix, S0).
isSigmaClass(Prefix, S0, S2) :-
	\+ isSigmaFormula(Prefix,S0),
	S1 is S0+1,
	isSigmaClass(Prefix,S1,S2).

%%% isSigmaFormula/isPiFormula(+Prefix, -Level)
%%%		computes the class level
%%%		+Prefix ... quantifier part
%%%		-Level  ... Class level according to the alternation of quantifiers
isSigmaFormula(F, 0) :-
	atomic(F).
isSigmaFormula('E' F, 1) :-
	atomic(F).
isSigmaFormula('E' (X 'A' F), S1) :-
	atomic(X),
	S1 > 0,
	S is S1-1,
	isPiFormula('A' F, S).
isSigmaFormula('E' (X 'E' F), S1) :-
	atomic(X),
	S1 > 0,
	isSigmaFormula('E' F, S1).

isPiFormula(F, 0) :-
	atomic(F).
isPiFormula('A' F, 1) :-
	atomic(F).
isPiFormula('A' X 'A' F, P1) :-
	atomic(X),
	P1 > 0,
	isPiFormula('A' F, P1).
isPiFormula('A' X 'E' F, P1) :-
	atomic(X),
	P1 > 0,
	P is P1-1,
	isSigmaFormula('E' F, P).

%%% checkClasses(+Class, -OK)
%%%		checks whether the algorithm works for all the class levels of the f.c.
%%%		i.e. whether all classes are s(1), p(1) or p(2)
%%%		Class ... List of class levels for the various frame conditions
%%%		OK ... 1 if OK, 0 if not
checkClasses([], 1).
checkClasses([H|T], OK) :-
	member(H, [s(1), p(1), p(2)]),
	checkClasses(T, OK).
checkClasses([H|_], OK) :-
	\+ member(H, [s(1), p(1), p(2)]),
	OK = 0.
