%%% Filename : f_completion.pl
%%% Date     : 06.02.2014
%%% Contents : converts a frame condition into a structural rule by applying the algorithm
%%%	       step(2): Rule Completion (i.e. Restructuring)
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% isRuleCompletion(+Consequent, -Structural)
%%	creates the structural rule from the given consequent // rule completion
%%	Consequent structure: 
%		[[[Antecedent1],[Succedent1]],[[A2],[S2]],...]
%%	Structural structure:
%%		[[[[PremiseA1], [PremiseS1]],[[PA2], [PS2]],...], [[CA], [CS]]], ...]
isRuleCompletion([],_,[]).
isRuleCompletion([[A,S]|T], C, Structural) :-
	member(C, [p(1)]),
	isRestructuring1([A,S], Restructured),
	isRuleCompletion(T, C, Structural1),
	append([Restructured], Structural1, Structural).
isRuleCompletion(Premises, C, Structural) :-
	member(C, [s(1),p(2)]),
	isRestructuring2(Premises, RestructuredPremises),
	Con = [['G'], ['D']],
	Structural = [[RestructuredPremises, Con]].
%	isRuleCompletion(T, C, Structural1),
%	append([Restructured], Structural1, Structural).


%% isRestructuring2(+Consequent, -Restructured)
%% isRestructuring1(+Consequent, -Restructured)
%%	restructures the consequent, i.e.:
%%		- isRestructuring2: puts all the formulas into the premises
%%		- isRestructuring1: puts only the formulas of the succedent into the
%%			premises; formulas of the antecedent are copied into the premises
%%	Consequent 	... consequent of the formula [antecedent, succedent]
%%	Restructured 	... the restructured formula [[Premises], [Consequent]]
isRestructuring2([], []).
isRestructuring2([[A,S]|T], RestructuredPremises) :-
	isRestructuring2(T, RestructuredPremises1),
	append(A, ['G'], PremA),
	append(['D'], S, PremS),
%	Con = [['G'], ['D']],
	append([[PremA, PremS]], RestructuredPremises1, RestructuredPremises).

isRestructuring1([A,S], Restructured):-
	append(A, S, PremA1),
	append(PremA1, ['G'], PremA),
	PremS = ['D'],
	append(A, ['G'], ConA),
	Con = [ConA, ['D']],
	Restructured = [[[PremA, PremS]], Con].
