%%% Filename : f_grammar.pl
%%% Date     : 04.02.2014
%%% Contents : DCG grammar for tex-output
%%% Author   : Lara Katharina Spendier <lara@logic.at>

bs        	  --> [92].
eol 	  	  --> [13,10].

%% Definitions for the style-file
texRule(C)		--> bs, "def", bs, "WRule{",C,"}",eol.
texRuleVerb(C)		--> bs, "def", bs, "WRuleV{",C,"}",eol.
texCondition(C)		--> bs, "def", bs, "WCondition{",C,"}",eol.
texConditionVerb(C)	--> bs, "def", bs, "WConditionV{",C,"}",eol.
texNameBaseCalculus(C) 	--> bs, "def", bs, "NameBaseCalculus{",C,"}",eol.
texFClass(C)  		--> bs, "def", bs, "FClass", "{$", bs, C, "$}", eol.
texNewFrameC(C)		--> bs, "def", bs, "NewFrameC", "{$", C, "$}", eol.
texNewFrameCNotMath(C)	--> bs, "def", bs, "NewFrameCNotMath", "{", C, "}", eol.
texNewFrameCList(C)	--> bs, "def", bs, "NewFrameCList", "{", bs, "begin{itemize}", bs, "item", C, bs, "end{itemize}}", eol.
texNewRuleStart   	--> bs, "def", bs, "NewRule", "{".
texNewRuleEnd	  	--> "}", eol.

%% tex-rewriting of the labelled rules with >= 0 premises

texNewAxC(C)      --> bs, "AxiomC{$",C,"$}",eol.
texNewUnC(C)	  --> bs, "UnaryInfC{$",C,"$}",eol.
texNoLine	  --> bs, "noLine", eol.

texNewUnCon(C)    --> bs, "UnaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.2cm}",eol.
texNewBinCon(C)   --> bs, "BinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.2cm}",eol.
texNewTriCon(C)   --> bs, "TrinaryInfC{$",C,"$}",bs,"DisplayProof",bs,"hspace*{0.3cm}",bs,"vspace*{0.2cm}",eol.

rule2tex([[H|T],[H1|T1]])  --> rule2tex([H|T]),bs,"Ra",rule2tex([H1|T1]).
rule2tex('G')     --> bs,"Gamma".
rule2tex('D')     --> bs,"Delta".
rule2tex('')      --> "".
rule2tex([])	  --> "".
rule2tex(X)	  --> frame2texA(X).
rule2tex(X)	  --> frame2tex(X).
rule2tex([H|[]])  --> rule2tex(H).
rule2tex([H|T])   --> rule2tex(H), ",", rule2tex(T).

nr2tex(N)	  --> {number_codes(N, Cs)}, Cs.

%%
%% frame2tex:   rewrites the axiom provided by the user to tex
%% frame2texP1: takes care of bracketing for formulas X &/v/* Y
%% frame2texP2: takes care of bracketing for formulas X -> Y
%%
frames2tex(X ';' Y)   --> frames2tex(X), ", ", frames2tex(Y).
frames2tex(X)	      --> frame2tex(X).
frames2texList(X ';' Y) --> frames2texList(X), bs, "item ", frames2texList(Y).
frames2texList(X)	--> "$", frame2tex(X), "$".
frame2tex(X ':' Y)    --> frame2texQ(X), " (", frame2texP(Y), ")".

frame2texP1(-X)       --> " ", bs, "lnot ", frame2texP1(X).
frame2texP1(X & Y)    --> "(", frame2texP2(X), " ", bs, "land ", frame2texP2(Y), ")".
frame2texP1(X v Y)    --> "(", frame2texP3(X), " ", bs, "lor ", frame2texP3(Y), ")".
frame2texP1(X -> Y)   --> "(", frame2texP5(X), " ", bs, "ra ", frame2texP5(Y), ")".
frame2texP1(X < Y)  --> {atomic(X),atomic(Y)}, frame2texV(X), bs, "leq ", frame2texV(Y).

frame2texP2(-X)       --> " ", bs, "lnot ", frame2texP1(X).
frame2texP2(X & Y)    --> frame2texP2(X), " ", bs, "land ", frame2texP2(Y).
frame2texP2(X v Y)    --> "(", frame2texP3(X), " ", bs, "lor ", frame2texP3(Y), ")".
frame2texP2(X -> Y)   --> "(", frame2texP5(X), " ", bs, "ra ", frame2texP5(Y), ")".
frame2texP2(X < Y)  --> {atomic(X),atomic(Y)}, frame2texV(X), bs, "leq ", frame2texV(Y).

frame2texP3(-X)       --> " ", bs, "lnot ", frame2texP1(X).
frame2texP3(X & Y)    --> "(", frame2texP2(X), " ", bs, "land ", frame2texP2(Y), ")".
frame2texP3(X v Y)    --> frame2texP3(X), " ", bs, "lor ", frame2texP3(Y).
frame2texP3(X -> Y)   --> "(", frame2texP5(X), " ", bs, "ra ", frame2texP5(Y), ")".
frame2texP3(X < Y)  --> {atomic(X),atomic(Y)}, frame2texV(X), bs, "leq ", frame2texV(Y).

frame2texP5(-X)       --> " ", bs, "lnot ", frame2texP1(X).
frame2texP5(X & Y)    --> frame2texP2(X), " ", bs, "land ", frame2texP2(Y).
frame2texP5(X v Y)    --> frame2texP3(X), " ", bs, "lor ", frame2texP3(Y).
frame2texP5(X -> Y)   --> "(", frame2texP5(X), " ", bs, "ra ", frame2texP5(Y), ")".
frame2texP5(X < Y)  --> {atomic(X),atomic(Y)}, frame2texV(X), bs, "leq ", frame2texV(Y).

frame2texQ('A' X)     --> " ", bs, "forall ", frame2texQ1(X).
frame2texQ('E' X)     --> " ", bs, "exists ", frame2texQ2(X).

frame2texQ1(X 'E' Y)  --> frame2texV(X), " ", bs, "exists ", frame2texQ2(Y).
frame2texQ1(X 'A' Y)  --> frame2texV(X), ",", frame2texQ1(Y).
frame2texQ1(X)	      --> frame2texV(X).
frame2texQ2(X 'E' Y)  --> frame2texV(X), ",", frame2texQ2(Y).
frame2texQ2(X 'A' Y)  --> frame2texV(X), " ", bs, "forall ", frame2texQ1(Y).
frame2texQ2(X)	      --> frame2texV(X).

frame2texP(-X) 	      --> " ", bs, "lnot ", frame2texP1(X).
frame2texP(X & Y)     --> frame2texP2(X), " ", bs, "land ", frame2texP2(Y).
frame2texP(X v Y)     --> frame2texP3(X), " ", bs, "lor ", frame2texP3(Y).
frame2texP(X -> Y)    --> frame2texP5(X), " ", bs, "ra ", frame2texP5(Y).
frame2texP(X)         --> frame2texA(X).

frame2texA(X < Y)  --> {atomic(X),atomic(Y)}, frame2texV(X),bs,"leq ",frame2texV(Y).

frame2texV(a) 	  --> "a".
frame2texV(b) 	  --> "b".
frame2texV(c)	  --> "c".
frame2texV(d)	  --> "d".
frame2texV(e)	  --> "e".
frame2texV(f)	  --> "f".
frame2texV(g)	  --> "g".
frame2texV(h)	  --> "h".
frame2texV(i)	  --> "i".
frame2texV(j)	  --> "j".
frame2texV(k)	  --> "k".
frame2texV(l)	  --> "l".
frame2texV(m)	  --> "m".
frame2texV(n)	  --> "n".
frame2texV(o)	  --> "o".
frame2texV(p)	  --> "p".
frame2texV(q)	  --> "q".
frame2texV(r)	  --> "r".
frame2texV(s)	  --> "s".
frame2texV(t)	  --> "t".
frame2texV(u)	  --> "u".
frame2texV(w)	  --> "w".
frame2texV(x)	  --> "x".
frame2texV(y)	  --> "y".
frame2texV(z)	  --> "z".

%frame2texV(a) 	  --> bs,"alpha".
%frame2texV(b) 	  --> bs,"beta".
%frame2texV(c)	  --> bs, "gamma".
%frame2texV(d)	  --> bs, "delta".
%frame2texV(e)	  --> bs, "varepsilon".
%frame2texV(f)	  --> bs, "zeta".
%frame2texV(g)	  --> bs, "eta".
%frame2texV(h)	  --> bs, "theta".
%frame2texV(i)	  --> bs, "iota".
%frame2texV(j)	  --> bs, "xi".
%frame2texV(k)	  --> bs, "kappa".
%frame2texV(l)	  --> bs, "lambda".
%frame2texV(m)	  --> bs, "mu".
%frame2texV(n)	  --> bs, "nu".
%frame2texV(o)	  --> bs, "omicron".
%frame2texV(p)	  --> bs, "pi".
%frame2texV(q)	  --> bs, "varpi".
%frame2texV(r)	  --> bs, "rho".
%frame2texV(s)	  --> bs, "sigma".
%frame2texV(t)	  --> bs, "tau".
%frame2texV(u)	  --> bs, "upsilon".
%frame2texV(w)	  --> bs, "psi".
%frame2texV(x)	  --> bs, "chi".
%frame2texV(y)	  --> bs, "psi".
%frame2texV(z)	  --> bs, "omega".

print_phrase(V) :-
   phrase(V, S),
   name(A, S),
   write(A).


