%%% Filename : f_printcmd.pl
%%% Date     : 07.02.2014
%%% Contents : prints the rules on the command-line
%%% Author   : Lara Katharina Spendier <lara@logic.at>


%% printrule(+Rule, +Web)
%%	prints the rule on the commandline; 
%% 	Rule ... [Premises, Conclusion]
%%	Web 	... 0 if started from cmdLine, 1 if started from web
printrule([], _).
printrule([[P, C]|T], Web) :-
	nl, nl, printpremises(P, Web),
	nl,write('-----------------------------'),nl,
	printconclusion(C, Web),
	printrule(T, Web).

%% printpremises(+Premise, +Web)
%%	prints the premises of the rule
%%	Web 	... 0 if started from cmdLine, 1 if started from web
printpremises([],_) :-
	write(' ').
printpremises([P|T], Web) :-
	printpremise(P, Web),
	printpremises(T, Web).

%% printpremise(+Premise, +Web)
%%	prints one premise on the command line
%%	Web 	... 0 if started from cmdLine, 1 if started from web
printpremise([A, S], Web) :-
	printantecedent(A, Web),
	write('=> '),
	printsuccedent(S, Web).

%% printconclusion(+Conclusion, +Web)
%%	prints the conclusion of the rule
%%	Web 	... 0 if started from cmdLine, 1 if started from web
printconclusion([A, S], Web) :-
	printantecedent(A, Web),
	write('=> '),
	printsuccedent(S, Web).

%% printantecedent(+Antecedent, +Web)
%%	prints the antecedent
%%	Web 	... 0 if started from cmdLine, 1 if started from web
printantecedent([], _).
printantecedent([H|[]], Web) :-
	( (Web = 0) ->
	  writef('%p ', [H])
	; ( (H = (A < B)) ->
	    writef('%p&lt;%p ', [A,B])
	  ; writef('%p ', [H])
	  )
	).
printantecedent([H|T], Web) :-
	( (Web = 0) ->
	  writef('%p,', [H])
	; ( (H = (A < B)) ->
	    writef('%p&lt;%p, ', [A,B])
	  ; writef('%p, ', [H])
	  )
	),
	printantecedent(T, Web).

%% printsuccedent(+Succedent, +Web)
%%	prints the succedent
%%	Web 	... 0 if started from cmdLine, 1 if started from web
printsuccedent([], _).
printsuccedent([H|[]], Web) :-
	( (Web = 0) ->
	  writef('%p      ', [H])
	; ( (H = (A < B)) ->
	    writef('%p&lt;%p      ', [A,B])
	  ; writef('%p      ', [H])
	  )
	).
printsuccedent([H|T], Web) :-
	( (Web = 0) ->
	  writef('%p,', [H])
	; ( (H = (A < B)) ->
	    writef('%p&lt;%p, ', [A,B])
	  ; writef('%p, ', [H])
	  )
	),
	printsuccedent(T, Web).
