%%% Filename : f_tex.pl
%%% Date     : 07.04.2014
%%% Contents : module for writing the output into a tex file
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% texOut(+FrameConditions, +Rules, +C)
%%	FrameConditions   ... Frame Conditions the user provided
%% 	Rules  	    	  ... Structural Rules for the frame condition ([[Premises, Conclusion],...])
%%	C 	    	  ... list of classes the f.c. are in
texOut(_,_,100) :-
	tell('ftex/Framinator.sty'),
	write('No'),
	told.

texOut(Frame, Rules, C) :-
	frames2tex(Frame,TFrame,[]),
	frames2texList(Frame, LFrame, []),
	tell('ftex/Framinator.sty'), 
	length(Rules, N),
	( (Frame=(_ ';' _)) ->
	  FN = 2
	; FN = 1
	),
	constants2tex(N, FN, C),
	print_phrase(texNewFrameC(TFrame)),
	print_phrase(texNewFrameCNotMath(TFrame)),
	print_phrase(texNewFrameCList(LFrame)),
	nl, print_phrase(texNewRuleStart),
	createSeqRules(Rules),
	print_phrase(texNewRuleEnd),
	told.

%% constants2tex(+BaseCalculus)
%%	prints the style information according to BaseCalculus to the style file
constants2tex(N, FN, C) :-
	( (FN = 1) ->
	  print_phrase(texCondition("condition")),
	  print_phrase(texConditionVerb("is"))
	; print_phrase(texCondition("conditions")),
	  print_phrase(texConditionVerb("are"))
	),
	( (N = 1) ->
	  print_phrase(texRule("rule")),
	  print_phrase(texRuleVerb("is"))
 	; print_phrase(texRule("rules")),
	  print_phrase(texRuleVerb("are"))
	),
	( (member(p(2), C)) ->
	  print_phrase(texFClass("Pi_2"))
	; ( (member(p(1),C),member(s(1),C)) ->
 	    print_phrase(texFClass("Pi_2"))
	  ; ( member(p(1),C) ->
	      print_phrase(texFClass("Pi_1"))
	    ; print_phrase(texFClass("Sigma_1"))
	    )
 	  )
	).

%% createSeqRule(+[Rules])
%% 	[Rules]  ... Several analytic rules for the class N2
createSeqRules([]).
createSeqRules([[Prem, Con]|T]) :-
	createSeqRule(Prem, Con),
	createSeqRules(T).

%% createSeqRule(+Prem, +Con)
%% 	Prem,Con  ... Analytic Sequent Rule for the axiom (Premises, Conclusion)
createSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [0,1]),
	rule2tex(Prem, TPrem, []),
	rule2tex(Con, TCon, []),
	print_phrase(texNewAxC(TPrem)),
	print_phrase(texNewUnCon(TCon)).

createSeqRule(Prem, Con) :-
	length(Prem,N),
	member(N, [2,4]),
	createPremises(Prem, TPrem),
	N1 is N/2+1,
	printBinPremises(0, TPrem, N1, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewBinCon(TCon)).

createSeqRule(Prem, Con) :-
	length(Prem,N),
	createPremises(Prem, TPrem),
	N1 is N//3,
	N2 is N mod 3,
	printTriPremises(0, TPrem, N1, N2, 1),
	rule2tex(Con, TCon, []),
	print_phrase(texNewTriCon(TCon)).

%% createPremises(+Prem,-TPrem])
%% 	Prem  ... Premises of the rule computed by AxiomCalc
%%	TPrem ... Premises of the rule in tex-format
createPremises([], []).
createPremises([H|T], TPrem) :-
	rule2tex(H, TH, []),
	createPremises(T, TPrem1),
	append([TH],TPrem1,TPrem).

%% printBinPremises(+Hyper, +Prem, +N, +I])
%%	for (hyper)sequent rules that have 2 or 4 premises
%% 	Hyper ... is 1 if hypersequent, 0 if sequent
%%	Prem  ... Premises of the rule
%%	N     ... number of premises
%%	I     ... counter
printBinPremises(_, [], _, _).

printBinPremises(Hyper, [H|T], N, I) :-
	member(I, [1,N]),
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printBinPremises(Hyper, T, N, I1).
	
printBinPremises(Hyper, [H|T], N, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printBinPremises(Hyper, T, N, I1).

%% printTriPremises(+Hyper, +Prem, +Div, +Mod, +I])
%%	for (hyper)sequent rules that have 3 or >4 premises
%% 	Hyper ... is 1 if hypersequent, 0 if sequent
%%	Prem  ... Premises of the rule
%%	Div   ... number of premises / 3 (Integer division)
%%	Mod   ... number of premises mod 3
%%		  --> to balance the number of premises in the rule!
%%	I     ... counter
printTriPremises(_, [], _, _, _).

printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	( Mod > 0 ->
 	  M1 is 1+Div+1
	; M1 is 1+Div
	), 
	( Mod > 1 ->
	  M2 is M1+Div+1
	; M2 is M1+Div
	),
	member(I, [1,M1,M2]),
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNewAxC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
	
printTriPremises(Hyper, [H|T], Div, Mod, I) :-
	I1 is I+1,
	( Hyper = 1 ->
	  append("G\\hh",H,H1)
	; H1 = H
	),
	print_phrase(texNoLine),
	print_phrase(texNewUnC(H1)),
	printTriPremises(Hyper, T, Div, Mod, I1).
